/*
 * Copyright (c) 2016, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.sun.glass.ui.monocle;

import java.util.HashMap;
import java.util.Map;

public class LinuxInputShim {

    public static final short INPUT_PROP_POINTER = LinuxInput.INPUT_PROP_POINTER;
    public static final short INPUT_PROP_DIRECT = LinuxInput.INPUT_PROP_DIRECT;
    public static final short INPUT_PROP_BUTTONPAD = LinuxInput.INPUT_PROP_BUTTONPAD;
    public static final short INPUT_PROP_SEMI_MT = LinuxInput.INPUT_PROP_SEMI_MT;
    public static final short INPUT_PROP_MAX = LinuxInput.INPUT_PROP_MAX;
    public static final short INPUT_PROP_CNT = LinuxInput.INPUT_PROP_CNT;
    public static final short EV_SYN = LinuxInput.EV_SYN;
    public static final short EV_KEY = LinuxInput.EV_KEY;
    public static final short EV_REL = LinuxInput.EV_REL;
    public static final short EV_ABS = LinuxInput.EV_ABS;
    public static final short EV_MSC = LinuxInput.EV_MSC;
    public static final short EV_SW = LinuxInput.EV_SW;
    public static final short EV_LED = LinuxInput.EV_LED;
    public static final short EV_SND = LinuxInput.EV_SND;
    public static final short EV_REP = LinuxInput.EV_REP;
    public static final short EV_FF = LinuxInput.EV_FF;
    public static final short EV_PWR = LinuxInput.EV_PWR;
    public static final short EV_FF_STATUS = LinuxInput.EV_FF_STATUS;
    public static final short EV_MAX = LinuxInput.EV_MAX;
    public static final short EV_CNT = LinuxInput.EV_CNT;
    public static final short SYN_REPORT = LinuxInput.SYN_REPORT;
    public static final short SYN_CONFIG = LinuxInput.SYN_CONFIG;
    public static final short SYN_MT_REPORT = LinuxInput.SYN_MT_REPORT;
    public static final short SYN_DROPPED = LinuxInput.SYN_DROPPED;
    public static final short KEY_RESERVED = LinuxInput.KEY_RESERVED;
    public static final short KEY_ESC = LinuxInput.KEY_ESC;
    public static final short KEY_1 = LinuxInput.KEY_1;
    public static final short KEY_2 = LinuxInput.KEY_2;
    public static final short KEY_3 = LinuxInput.KEY_3;
    public static final short KEY_4 = LinuxInput.KEY_4;
    public static final short KEY_5 = LinuxInput.KEY_5;
    public static final short KEY_6 = LinuxInput.KEY_6;
    public static final short KEY_7 = LinuxInput.KEY_7;
    public static final short KEY_8 = LinuxInput.KEY_8;
    public static final short KEY_9 = LinuxInput.KEY_9;
    public static final short KEY_0 = LinuxInput.KEY_0;
    public static final short KEY_MINUS = LinuxInput.KEY_MINUS;
    public static final short KEY_EQUAL = LinuxInput.KEY_EQUAL;
    public static final short KEY_BACKSPACE = LinuxInput.KEY_BACKSPACE;
    public static final short KEY_TAB = LinuxInput.KEY_TAB;
    public static final short KEY_Q = LinuxInput.KEY_Q;
    public static final short KEY_W = LinuxInput.KEY_W;
    public static final short KEY_E = LinuxInput.KEY_E;
    public static final short KEY_R = LinuxInput.KEY_R;
    public static final short KEY_T = LinuxInput.KEY_T;
    public static final short KEY_Y = LinuxInput.KEY_Y;
    public static final short KEY_U = LinuxInput.KEY_U;
    public static final short KEY_I = LinuxInput.KEY_I;
    public static final short KEY_O = LinuxInput.KEY_O;
    public static final short KEY_P = LinuxInput.KEY_P;
    public static final short KEY_LEFTBRACE = LinuxInput.KEY_LEFTBRACE;
    public static final short KEY_RIGHTBRACE = LinuxInput.KEY_RIGHTBRACE;
    public static final short KEY_ENTER = LinuxInput.KEY_ENTER;
    public static final short KEY_LEFTCTRL = LinuxInput.KEY_LEFTCTRL;
    public static final short KEY_A = LinuxInput.KEY_A;
    public static final short KEY_S = LinuxInput.KEY_S;
    public static final short KEY_D = LinuxInput.KEY_D;
    public static final short KEY_F = LinuxInput.KEY_F;
    public static final short KEY_G = LinuxInput.KEY_G;
    public static final short KEY_H = LinuxInput.KEY_H;
    public static final short KEY_J = LinuxInput.KEY_J;
    public static final short KEY_K = LinuxInput.KEY_K;
    public static final short KEY_L = LinuxInput.KEY_L;
    public static final short KEY_SEMICOLON = LinuxInput.KEY_SEMICOLON;
    public static final short KEY_APOSTROPHE = LinuxInput.KEY_APOSTROPHE;
    public static final short KEY_GRAVE = LinuxInput.KEY_GRAVE;
    public static final short KEY_LEFTSHIFT = LinuxInput.KEY_LEFTSHIFT;
    public static final short KEY_BACKSLASH = LinuxInput.KEY_BACKSLASH;
    public static final short KEY_Z = LinuxInput.KEY_Z;
    public static final short KEY_X = LinuxInput.KEY_X;
    public static final short KEY_C = LinuxInput.KEY_C;
    public static final short KEY_V = LinuxInput.KEY_V;
    public static final short KEY_B = LinuxInput.KEY_B;
    public static final short KEY_N = LinuxInput.KEY_N;
    public static final short KEY_M = LinuxInput.KEY_M;
    public static final short KEY_COMMA = LinuxInput.KEY_COMMA;
    public static final short KEY_DOT = LinuxInput.KEY_DOT;
    public static final short KEY_SLASH = LinuxInput.KEY_SLASH;
    public static final short KEY_RIGHTSHIFT = LinuxInput.KEY_RIGHTSHIFT;
    public static final short KEY_KPASTERISK = LinuxInput.KEY_KPASTERISK;
    public static final short KEY_LEFTALT = LinuxInput.KEY_LEFTALT;
    public static final short KEY_SPACE = LinuxInput.KEY_SPACE;
    public static final short KEY_CAPSLOCK = LinuxInput.KEY_CAPSLOCK;
    public static final short KEY_F1 = LinuxInput.KEY_F1;
    public static final short KEY_F2 = LinuxInput.KEY_F2;
    public static final short KEY_F3 = LinuxInput.KEY_F3;
    public static final short KEY_F4 = LinuxInput.KEY_F4;
    public static final short KEY_F5 = LinuxInput.KEY_F5;
    public static final short KEY_F6 = LinuxInput.KEY_F6;
    public static final short KEY_F7 = LinuxInput.KEY_F7;
    public static final short KEY_F8 = LinuxInput.KEY_F8;
    public static final short KEY_F9 = LinuxInput.KEY_F9;
    public static final short KEY_F10 = LinuxInput.KEY_F10;
    public static final short KEY_NUMLOCK = LinuxInput.KEY_NUMLOCK;
    public static final short KEY_SCROLLLOCK = LinuxInput.KEY_SCROLLLOCK;
    public static final short KEY_KP7 = LinuxInput.KEY_KP7;
    public static final short KEY_KP8 = LinuxInput.KEY_KP8;
    public static final short KEY_KP9 = LinuxInput.KEY_KP9;
    public static final short KEY_KPMINUS = LinuxInput.KEY_KPMINUS;
    public static final short KEY_KP4 = LinuxInput.KEY_KP4;
    public static final short KEY_KP5 = LinuxInput.KEY_KP5;
    public static final short KEY_KP6 = LinuxInput.KEY_KP6;
    public static final short KEY_KPPLUS = LinuxInput.KEY_KPPLUS;
    public static final short KEY_KP1 = LinuxInput.KEY_KP1;
    public static final short KEY_KP2 = LinuxInput.KEY_KP2;
    public static final short KEY_KP3 = LinuxInput.KEY_KP3;
    public static final short KEY_KP0 = LinuxInput.KEY_KP0;
    public static final short KEY_KPDOT = LinuxInput.KEY_KPDOT;
    public static final short KEY_ZENKAKUHANKAKU = LinuxInput.KEY_ZENKAKUHANKAKU;
    public static final short KEY_102ND = LinuxInput.KEY_102ND;
    public static final short KEY_F11 = LinuxInput.KEY_F11;
    public static final short KEY_F12 = LinuxInput.KEY_F12;
    public static final short KEY_RO = LinuxInput.KEY_RO;
    public static final short KEY_KATAKANA = LinuxInput.KEY_KATAKANA;
    public static final short KEY_HIRAGANA = LinuxInput.KEY_HIRAGANA;
    public static final short KEY_HENKAN = LinuxInput.KEY_HENKAN;
    public static final short KEY_KATAKANAHIRAGANA = LinuxInput.KEY_KATAKANAHIRAGANA;
    public static final short KEY_MUHENKAN = LinuxInput.KEY_MUHENKAN;
    public static final short KEY_KPJPCOMMA = LinuxInput.KEY_KPJPCOMMA;
    public static final short KEY_KPENTER = LinuxInput.KEY_KPENTER;
    public static final short KEY_RIGHTCTRL = LinuxInput.KEY_RIGHTCTRL;
    public static final short KEY_KPSLASH = LinuxInput.KEY_KPSLASH;
    public static final short KEY_SYSRQ = LinuxInput.KEY_SYSRQ;
    public static final short KEY_RIGHTALT = LinuxInput.KEY_RIGHTALT;
    public static final short KEY_LINEFEED = LinuxInput.KEY_LINEFEED;
    public static final short KEY_HOME = LinuxInput.KEY_HOME;
    public static final short KEY_UP = LinuxInput.KEY_UP;
    public static final short KEY_PAGEUP = LinuxInput.KEY_PAGEUP;
    public static final short KEY_LEFT = LinuxInput.KEY_LEFT;
    public static final short KEY_RIGHT = LinuxInput.KEY_RIGHT;
    public static final short KEY_END = LinuxInput.KEY_END;
    public static final short KEY_DOWN = LinuxInput.KEY_DOWN;
    public static final short KEY_PAGEDOWN = LinuxInput.KEY_PAGEDOWN;
    public static final short KEY_INSERT = LinuxInput.KEY_INSERT;
    public static final short KEY_DELETE = LinuxInput.KEY_DELETE;
    public static final short KEY_MACRO = LinuxInput.KEY_MACRO;
    public static final short KEY_MUTE = LinuxInput.KEY_MUTE;
    public static final short KEY_VOLUMEDOWN = LinuxInput.KEY_VOLUMEDOWN;
    public static final short KEY_VOLUMEUP = LinuxInput.KEY_VOLUMEUP;
    public static final short KEY_POWER = LinuxInput.KEY_POWER;
    public static final short KEY_KPEQUAL = LinuxInput.KEY_KPEQUAL;
    public static final short KEY_KPPLUSMINUS = LinuxInput.KEY_KPPLUSMINUS;
    public static final short KEY_PAUSE = LinuxInput.KEY_PAUSE;
    public static final short KEY_SCALE = LinuxInput.KEY_SCALE;
    public static final short KEY_KPCOMMA = LinuxInput.KEY_KPCOMMA;
    public static final short KEY_HANGEUL = LinuxInput.KEY_HANGEUL;
    public static final short KEY_HANGUEL = LinuxInput.KEY_HANGUEL;
    public static final short KEY_HANJA = LinuxInput.KEY_HANJA;
    public static final short KEY_YEN = LinuxInput.KEY_YEN;
    public static final short KEY_LEFTMETA = LinuxInput.KEY_LEFTMETA;
    public static final short KEY_RIGHTMETA = LinuxInput.KEY_RIGHTMETA;
    public static final short KEY_COMPOSE = LinuxInput.KEY_COMPOSE;
    public static final short KEY_STOP = LinuxInput.KEY_STOP;
    public static final short KEY_AGAIN = LinuxInput.KEY_AGAIN;
    public static final short KEY_PROPS = LinuxInput.KEY_PROPS;
    public static final short KEY_UNDO = LinuxInput.KEY_UNDO;
    public static final short KEY_FRONT = LinuxInput.KEY_FRONT;
    public static final short KEY_COPY = LinuxInput.KEY_COPY;
    public static final short KEY_OPEN = LinuxInput.KEY_OPEN;
    public static final short KEY_PASTE = LinuxInput.KEY_PASTE;
    public static final short KEY_FIND = LinuxInput.KEY_FIND;
    public static final short KEY_CUT = LinuxInput.KEY_CUT;
    public static final short KEY_HELP = LinuxInput.KEY_HELP;
    public static final short KEY_MENU = LinuxInput.KEY_MENU;
    public static final short KEY_CALC = LinuxInput.KEY_CALC;
    public static final short KEY_SETUP = LinuxInput.KEY_SETUP;
    public static final short KEY_SLEEP = LinuxInput.KEY_SLEEP;
    public static final short KEY_WAKEUP = LinuxInput.KEY_WAKEUP;
    public static final short KEY_FILE = LinuxInput.KEY_FILE;
    public static final short KEY_SENDFILE = LinuxInput.KEY_SENDFILE;
    public static final short KEY_DELETEFILE = LinuxInput.KEY_DELETEFILE;
    public static final short KEY_XFER = LinuxInput.KEY_XFER;
    public static final short KEY_PROG1 = LinuxInput.KEY_PROG1;
    public static final short KEY_PROG2 = LinuxInput.KEY_PROG2;
    public static final short KEY_WWW = LinuxInput.KEY_WWW;
    public static final short KEY_MSDOS = LinuxInput.KEY_MSDOS;
    public static final short KEY_COFFEE = LinuxInput.KEY_COFFEE;
    public static final short KEY_SCREENLOCK = LinuxInput.KEY_SCREENLOCK;
    public static final short KEY_DIRECTION = LinuxInput.KEY_DIRECTION;
    public static final short KEY_CYCLEWINDOWS = LinuxInput.KEY_CYCLEWINDOWS;
    public static final short KEY_MAIL = LinuxInput.KEY_MAIL;
    public static final short KEY_BOOKMARKS = LinuxInput.KEY_BOOKMARKS;
    public static final short KEY_COMPUTER = LinuxInput.KEY_COMPUTER;
    public static final short KEY_BACK = LinuxInput.KEY_BACK;
    public static final short KEY_FORWARD = LinuxInput.KEY_FORWARD;
    public static final short KEY_CLOSECD = LinuxInput.KEY_CLOSECD;
    public static final short KEY_EJECTCD = LinuxInput.KEY_EJECTCD;
    public static final short KEY_EJECTCLOSECD = LinuxInput.KEY_EJECTCLOSECD;
    public static final short KEY_NEXTSONG = LinuxInput.KEY_NEXTSONG;
    public static final short KEY_PLAYPAUSE = LinuxInput.KEY_PLAYPAUSE;
    public static final short KEY_PREVIOUSSONG = LinuxInput.KEY_PREVIOUSSONG;
    public static final short KEY_STOPCD = LinuxInput.KEY_STOPCD;
    public static final short KEY_RECORD = LinuxInput.KEY_RECORD;
    public static final short KEY_REWIND = LinuxInput.KEY_REWIND;
    public static final short KEY_PHONE = LinuxInput.KEY_PHONE;
    public static final short KEY_ISO = LinuxInput.KEY_ISO;
    public static final short KEY_CONFIG = LinuxInput.KEY_CONFIG;
    public static final short KEY_HOMEPAGE = LinuxInput.KEY_HOMEPAGE;
    public static final short KEY_REFRESH = LinuxInput.KEY_REFRESH;
    public static final short KEY_EXIT = LinuxInput.KEY_EXIT;
    public static final short KEY_MOVE = LinuxInput.KEY_MOVE;
    public static final short KEY_EDIT = LinuxInput.KEY_EDIT;
    public static final short KEY_SCROLLUP = LinuxInput.KEY_SCROLLUP;
    public static final short KEY_SCROLLDOWN = LinuxInput.KEY_SCROLLDOWN;
    public static final short KEY_KPLEFTPAREN = LinuxInput.KEY_KPLEFTPAREN;
    public static final short KEY_KPRIGHTPAREN = LinuxInput.KEY_KPRIGHTPAREN;
    public static final short KEY_NEW = LinuxInput.KEY_NEW;
    public static final short KEY_REDO = LinuxInput.KEY_REDO;
    public static final short KEY_F13 = LinuxInput.KEY_F13;
    public static final short KEY_F14 = LinuxInput.KEY_F14;
    public static final short KEY_F15 = LinuxInput.KEY_F15;
    public static final short KEY_F16 = LinuxInput.KEY_F16;
    public static final short KEY_F17 = LinuxInput.KEY_F17;
    public static final short KEY_F18 = LinuxInput.KEY_F18;
    public static final short KEY_F19 = LinuxInput.KEY_F19;
    public static final short KEY_F20 = LinuxInput.KEY_F20;
    public static final short KEY_F21 = LinuxInput.KEY_F21;
    public static final short KEY_F22 = LinuxInput.KEY_F22;
    public static final short KEY_F23 = LinuxInput.KEY_F23;
    public static final short KEY_F24 = LinuxInput.KEY_F24;
    public static final short KEY_PLAYCD = LinuxInput.KEY_PLAYCD;
    public static final short KEY_PAUSECD = LinuxInput.KEY_PAUSECD;
    public static final short KEY_PROG3 = LinuxInput.KEY_PROG3;
    public static final short KEY_PROG4 = LinuxInput.KEY_PROG4;
    public static final short KEY_DASHBOARD = LinuxInput.KEY_DASHBOARD;
    public static final short KEY_SUSPEND = LinuxInput.KEY_SUSPEND;
    public static final short KEY_CLOSE = LinuxInput.KEY_CLOSE;
    public static final short KEY_PLAY = LinuxInput.KEY_PLAY;
    public static final short KEY_FASTFORWARD = LinuxInput.KEY_FASTFORWARD;
    public static final short KEY_BASSBOOST = LinuxInput.KEY_BASSBOOST;
    public static final short KEY_PRINT = LinuxInput.KEY_PRINT;
    public static final short KEY_HP = LinuxInput.KEY_HP;
    public static final short KEY_CAMERA = LinuxInput.KEY_CAMERA;
    public static final short KEY_SOUND = LinuxInput.KEY_SOUND;
    public static final short KEY_QUESTION = LinuxInput.KEY_QUESTION;
    public static final short KEY_EMAIL = LinuxInput.KEY_EMAIL;
    public static final short KEY_CHAT = LinuxInput.KEY_CHAT;
    public static final short KEY_SEARCH = LinuxInput.KEY_SEARCH;
    public static final short KEY_CONNECT = LinuxInput.KEY_CONNECT;
    public static final short KEY_FINANCE = LinuxInput.KEY_FINANCE;
    public static final short KEY_SPORT = LinuxInput.KEY_SPORT;
    public static final short KEY_SHOP = LinuxInput.KEY_SHOP;
    public static final short KEY_ALTERASE = LinuxInput.KEY_ALTERASE;
    public static final short KEY_CANCEL = LinuxInput.KEY_CANCEL;
    public static final short KEY_BRIGHTNESSDOWN = LinuxInput.KEY_BRIGHTNESSDOWN;
    public static final short KEY_BRIGHTNESSUP = LinuxInput.KEY_BRIGHTNESSUP;
    public static final short KEY_MEDIA = LinuxInput.KEY_MEDIA;
    public static final short KEY_SWITCHVIDEOMODE = LinuxInput.KEY_SWITCHVIDEOMODE;
    public static final short KEY_KBDILLUMTOGGLE = LinuxInput.KEY_KBDILLUMTOGGLE;
    public static final short KEY_KBDILLUMDOWN = LinuxInput.KEY_KBDILLUMDOWN;
    public static final short KEY_KBDILLUMUP = LinuxInput.KEY_KBDILLUMUP;
    public static final short KEY_SEND = LinuxInput.KEY_SEND;
    public static final short KEY_REPLY = LinuxInput.KEY_REPLY;
    public static final short KEY_FORWARDMAIL = LinuxInput.KEY_FORWARDMAIL;
    public static final short KEY_SAVE = LinuxInput.KEY_SAVE;
    public static final short KEY_DOCUMENTS = LinuxInput.KEY_DOCUMENTS;
    public static final short KEY_BATTERY = LinuxInput.KEY_BATTERY;
    public static final short KEY_BLUETOOTH = LinuxInput.KEY_BLUETOOTH;
    public static final short KEY_WLAN = LinuxInput.KEY_WLAN;
    public static final short KEY_UWB = LinuxInput.KEY_UWB;
    public static final short KEY_UNKNOWN = LinuxInput.KEY_UNKNOWN;
    public static final short KEY_VIDEO_NEXT = LinuxInput.KEY_VIDEO_NEXT;
    public static final short KEY_VIDEO_PREV = LinuxInput.KEY_VIDEO_PREV;
    public static final short KEY_BRIGHTNESS_CYCLE = LinuxInput.KEY_BRIGHTNESS_CYCLE;
    public static final short KEY_BRIGHTNESS_ZERO = LinuxInput.KEY_BRIGHTNESS_ZERO;
    public static final short KEY_DISPLAY_OFF = LinuxInput.KEY_DISPLAY_OFF;
    public static final short KEY_WIMAX = LinuxInput.KEY_WIMAX;
    public static final short KEY_RFKILL = LinuxInput.KEY_RFKILL;
    public static final short KEY_MICMUTE = LinuxInput.KEY_MICMUTE;
    public static final short BTN_MISC = LinuxInput.BTN_MISC;
    public static final short BTN_0 = LinuxInput.BTN_0;
    public static final short BTN_1 = LinuxInput.BTN_1;
    public static final short BTN_2 = LinuxInput.BTN_2;
    public static final short BTN_3 = LinuxInput.BTN_3;
    public static final short BTN_4 = LinuxInput.BTN_4;
    public static final short BTN_5 = LinuxInput.BTN_5;
    public static final short BTN_6 = LinuxInput.BTN_6;
    public static final short BTN_7 = LinuxInput.BTN_7;
    public static final short BTN_8 = LinuxInput.BTN_8;
    public static final short BTN_9 = LinuxInput.BTN_9;
    public static final short BTN_MOUSE = LinuxInput.BTN_MOUSE;
    public static final short BTN_LEFT = LinuxInput.BTN_LEFT;
    public static final short BTN_RIGHT = LinuxInput.BTN_RIGHT;
    public static final short BTN_MIDDLE = LinuxInput.BTN_MIDDLE;
    public static final short BTN_SIDE = LinuxInput.BTN_SIDE;
    public static final short BTN_EXTRA = LinuxInput.BTN_EXTRA;
    public static final short BTN_FORWARD = LinuxInput.BTN_FORWARD;
    public static final short BTN_BACK = LinuxInput.BTN_BACK;
    public static final short BTN_TASK = LinuxInput.BTN_TASK;
    public static final short BTN_JOYSTICK = LinuxInput.BTN_JOYSTICK;
    public static final short BTN_TRIGGER = LinuxInput.BTN_TRIGGER;
    public static final short BTN_THUMB = LinuxInput.BTN_THUMB;
    public static final short BTN_THUMB2 = LinuxInput.BTN_THUMB2;
    public static final short BTN_TOP = LinuxInput.BTN_TOP;
    public static final short BTN_TOP2 = LinuxInput.BTN_TOP2;
    public static final short BTN_PINKIE = LinuxInput.BTN_PINKIE;
    public static final short BTN_BASE = LinuxInput.BTN_BASE;
    public static final short BTN_BASE2 = LinuxInput.BTN_BASE2;
    public static final short BTN_BASE3 = LinuxInput.BTN_BASE3;
    public static final short BTN_BASE4 = LinuxInput.BTN_BASE4;
    public static final short BTN_BASE5 = LinuxInput.BTN_BASE5;
    public static final short BTN_BASE6 = LinuxInput.BTN_BASE6;
    public static final short BTN_DEAD = LinuxInput.BTN_DEAD;
    public static final short BTN_GAMEPAD = LinuxInput.BTN_GAMEPAD;
    public static final short BTN_A = LinuxInput.BTN_A;
    public static final short BTN_B = LinuxInput.BTN_B;
    public static final short BTN_C = LinuxInput.BTN_C;
    public static final short BTN_X = LinuxInput.BTN_X;
    public static final short BTN_Y = LinuxInput.BTN_Y;
    public static final short BTN_Z = LinuxInput.BTN_Z;
    public static final short BTN_TL = LinuxInput.BTN_TL;
    public static final short BTN_TR = LinuxInput.BTN_TR;
    public static final short BTN_TL2 = LinuxInput.BTN_TL2;
    public static final short BTN_TR2 = LinuxInput.BTN_TR2;
    public static final short BTN_SELECT = LinuxInput.BTN_SELECT;
    public static final short BTN_START = LinuxInput.BTN_START;
    public static final short BTN_MODE = LinuxInput.BTN_MODE;
    public static final short BTN_THUMBL = LinuxInput.BTN_THUMBL;
    public static final short BTN_THUMBR = LinuxInput.BTN_THUMBR;
    public static final short BTN_DIGI = LinuxInput.BTN_DIGI;
    public static final short BTN_TOOL_PEN = LinuxInput.BTN_TOOL_PEN;
    public static final short BTN_TOOL_RUBBER = LinuxInput.BTN_TOOL_RUBBER;
    public static final short BTN_TOOL_BRUSH = LinuxInput.BTN_TOOL_BRUSH;
    public static final short BTN_TOOL_PENCIL = LinuxInput.BTN_TOOL_PENCIL;
    public static final short BTN_TOOL_AIRBRUSH = LinuxInput.BTN_TOOL_AIRBRUSH;
    public static final short BTN_TOOL_FINGER = LinuxInput.BTN_TOOL_FINGER;
    public static final short BTN_TOOL_MOUSE = LinuxInput.BTN_TOOL_MOUSE;
    public static final short BTN_TOOL_LENS = LinuxInput.BTN_TOOL_LENS;
    public static final short BTN_TOOL_QUINTTAP = LinuxInput.BTN_TOOL_QUINTTAP;
    public static final short BTN_TOUCH = LinuxInput.BTN_TOUCH;
    public static final short BTN_STYLUS = LinuxInput.BTN_STYLUS;
    public static final short BTN_STYLUS2 = LinuxInput.BTN_STYLUS2;
    public static final short BTN_TOOL_DOUBLETAP = LinuxInput.BTN_TOOL_DOUBLETAP;
    public static final short BTN_TOOL_TRIPLETAP = LinuxInput.BTN_TOOL_TRIPLETAP;
    public static final short BTN_TOOL_QUADTAP = LinuxInput.BTN_TOOL_QUADTAP;
    public static final short BTN_WHEEL = LinuxInput.BTN_WHEEL;
    public static final short BTN_GEAR_DOWN = LinuxInput.BTN_GEAR_DOWN;
    public static final short BTN_GEAR_UP = LinuxInput.BTN_GEAR_UP;
    public static final short KEY_OK = LinuxInput.KEY_OK;
    public static final short KEY_SELECT = LinuxInput.KEY_SELECT;
    public static final short KEY_GOTO = LinuxInput.KEY_GOTO;
    public static final short KEY_CLEAR = LinuxInput.KEY_CLEAR;
    public static final short KEY_POWER2 = LinuxInput.KEY_POWER2;
    public static final short KEY_OPTION = LinuxInput.KEY_OPTION;
    public static final short KEY_INFO = LinuxInput.KEY_INFO;
    public static final short KEY_TIME = LinuxInput.KEY_TIME;
    public static final short KEY_VENDOR = LinuxInput.KEY_VENDOR;
    public static final short KEY_ARCHIVE = LinuxInput.KEY_ARCHIVE;
    public static final short KEY_PROGRAM = LinuxInput.KEY_PROGRAM;
    public static final short KEY_CHANNEL = LinuxInput.KEY_CHANNEL;
    public static final short KEY_FAVORITES = LinuxInput.KEY_FAVORITES;
    public static final short KEY_EPG = LinuxInput.KEY_EPG;
    public static final short KEY_PVR = LinuxInput.KEY_PVR;
    public static final short KEY_MHP = LinuxInput.KEY_MHP;
    public static final short KEY_LANGUAGE = LinuxInput.KEY_LANGUAGE;
    public static final short KEY_TITLE = LinuxInput.KEY_TITLE;
    public static final short KEY_SUBTITLE = LinuxInput.KEY_SUBTITLE;
    public static final short KEY_ANGLE = LinuxInput.KEY_ANGLE;
    public static final short KEY_ZOOM = LinuxInput.KEY_ZOOM;
    public static final short KEY_MODE = LinuxInput.KEY_MODE;
    public static final short KEY_KEYBOARD = LinuxInput.KEY_KEYBOARD;
    public static final short KEY_SCREEN = LinuxInput.KEY_SCREEN;
    public static final short KEY_PC = LinuxInput.KEY_PC;
    public static final short KEY_TV = LinuxInput.KEY_TV;
    public static final short KEY_TV2 = LinuxInput.KEY_TV2;
    public static final short KEY_VCR = LinuxInput.KEY_VCR;
    public static final short KEY_VCR2 = LinuxInput.KEY_VCR2;
    public static final short KEY_SAT = LinuxInput.KEY_SAT;
    public static final short KEY_SAT2 = LinuxInput.KEY_SAT2;
    public static final short KEY_CD = LinuxInput.KEY_CD;
    public static final short KEY_TAPE = LinuxInput.KEY_TAPE;
    public static final short KEY_RADIO = LinuxInput.KEY_RADIO;
    public static final short KEY_TUNER = LinuxInput.KEY_TUNER;
    public static final short KEY_PLAYER = LinuxInput.KEY_PLAYER;
    public static final short KEY_TEXT = LinuxInput.KEY_TEXT;
    public static final short KEY_DVD = LinuxInput.KEY_DVD;
    public static final short KEY_AUX = LinuxInput.KEY_AUX;
    public static final short KEY_MP3 = LinuxInput.KEY_MP3;
    public static final short KEY_AUDIO = LinuxInput.KEY_AUDIO;
    public static final short KEY_VIDEO = LinuxInput.KEY_VIDEO;
    public static final short KEY_DIRECTORY = LinuxInput.KEY_DIRECTORY;
    public static final short KEY_LIST = LinuxInput.KEY_LIST;
    public static final short KEY_MEMO = LinuxInput.KEY_MEMO;
    public static final short KEY_CALENDAR = LinuxInput.KEY_CALENDAR;
    public static final short KEY_RED = LinuxInput.KEY_RED;
    public static final short KEY_GREEN = LinuxInput.KEY_GREEN;
    public static final short KEY_YELLOW = LinuxInput.KEY_YELLOW;
    public static final short KEY_BLUE = LinuxInput.KEY_BLUE;
    public static final short KEY_CHANNELUP = LinuxInput.KEY_CHANNELUP;
    public static final short KEY_CHANNELDOWN = LinuxInput.KEY_CHANNELDOWN;
    public static final short KEY_FIRST = LinuxInput.KEY_FIRST;
    public static final short KEY_LAST = LinuxInput.KEY_LAST;
    public static final short KEY_AB = LinuxInput.KEY_AB;
    public static final short KEY_NEXT = LinuxInput.KEY_NEXT;
    public static final short KEY_RESTART = LinuxInput.KEY_RESTART;
    public static final short KEY_SLOW = LinuxInput.KEY_SLOW;
    public static final short KEY_SHUFFLE = LinuxInput.KEY_SHUFFLE;
    public static final short KEY_BREAK = LinuxInput.KEY_BREAK;
    public static final short KEY_PREVIOUS = LinuxInput.KEY_PREVIOUS;
    public static final short KEY_DIGITS = LinuxInput.KEY_DIGITS;
    public static final short KEY_TEEN = LinuxInput.KEY_TEEN;
    public static final short KEY_TWEN = LinuxInput.KEY_TWEN;
    public static final short KEY_VIDEOPHONE = LinuxInput.KEY_VIDEOPHONE;
    public static final short KEY_GAMES = LinuxInput.KEY_GAMES;
    public static final short KEY_ZOOMIN = LinuxInput.KEY_ZOOMIN;
    public static final short KEY_ZOOMOUT = LinuxInput.KEY_ZOOMOUT;
    public static final short KEY_ZOOMRESET = LinuxInput.KEY_ZOOMRESET;
    public static final short KEY_WORDPROCESSOR = LinuxInput.KEY_WORDPROCESSOR;
    public static final short KEY_EDITOR = LinuxInput.KEY_EDITOR;
    public static final short KEY_SPREADSHEET = LinuxInput.KEY_SPREADSHEET;
    public static final short KEY_GRAPHICSEDITOR = LinuxInput.KEY_GRAPHICSEDITOR;
    public static final short KEY_PRESENTATION = LinuxInput.KEY_PRESENTATION;
    public static final short KEY_DATABASE = LinuxInput.KEY_DATABASE;
    public static final short KEY_NEWS = LinuxInput.KEY_NEWS;
    public static final short KEY_VOICEMAIL = LinuxInput.KEY_VOICEMAIL;
    public static final short KEY_ADDRESSBOOK = LinuxInput.KEY_ADDRESSBOOK;
    public static final short KEY_MESSENGER = LinuxInput.KEY_MESSENGER;
    public static final short KEY_DISPLAYTOGGLE = LinuxInput.KEY_DISPLAYTOGGLE;
    public static final short KEY_SPELLCHECK = LinuxInput.KEY_SPELLCHECK;
    public static final short KEY_LOGOFF = LinuxInput.KEY_LOGOFF;
    public static final short KEY_DOLLAR = LinuxInput.KEY_DOLLAR;
    public static final short KEY_EURO = LinuxInput.KEY_EURO;
    public static final short KEY_FRAMEBACK = LinuxInput.KEY_FRAMEBACK;
    public static final short KEY_FRAMEFORWARD = LinuxInput.KEY_FRAMEFORWARD;
    public static final short KEY_CONTEXT_MENU = LinuxInput.KEY_CONTEXT_MENU;
    public static final short KEY_MEDIA_REPEAT = LinuxInput.KEY_MEDIA_REPEAT;
    public static final short KEY_10CHANNELSUP = LinuxInput.KEY_10CHANNELSUP;
    public static final short KEY_10CHANNELSDOWN = LinuxInput.KEY_10CHANNELSDOWN;
    public static final short KEY_IMAGES = LinuxInput.KEY_IMAGES;
    public static final short KEY_DEL_EOL = LinuxInput.KEY_DEL_EOL;
    public static final short KEY_DEL_EOS = LinuxInput.KEY_DEL_EOS;
    public static final short KEY_INS_LINE = LinuxInput.KEY_INS_LINE;
    public static final short KEY_DEL_LINE = LinuxInput.KEY_DEL_LINE;
    public static final short KEY_FN = LinuxInput.KEY_FN;
    public static final short KEY_FN_ESC = LinuxInput.KEY_FN_ESC;
    public static final short KEY_FN_F1 = LinuxInput.KEY_FN_F1;
    public static final short KEY_FN_F2 = LinuxInput.KEY_FN_F2;
    public static final short KEY_FN_F3 = LinuxInput.KEY_FN_F3;
    public static final short KEY_FN_F4 = LinuxInput.KEY_FN_F4;
    public static final short KEY_FN_F5 = LinuxInput.KEY_FN_F5;
    public static final short KEY_FN_F6 = LinuxInput.KEY_FN_F6;
    public static final short KEY_FN_F7 = LinuxInput.KEY_FN_F7;
    public static final short KEY_FN_F8 = LinuxInput.KEY_FN_F8;
    public static final short KEY_FN_F9 = LinuxInput.KEY_FN_F9;
    public static final short KEY_FN_F10 = LinuxInput.KEY_FN_F10;
    public static final short KEY_FN_F11 = LinuxInput.KEY_FN_F11;
    public static final short KEY_FN_F12 = LinuxInput.KEY_FN_F12;
    public static final short KEY_FN_1 = LinuxInput.KEY_FN_1;
    public static final short KEY_FN_2 = LinuxInput.KEY_FN_2;
    public static final short KEY_FN_D = LinuxInput.KEY_FN_D;
    public static final short KEY_FN_E = LinuxInput.KEY_FN_E;
    public static final short KEY_FN_F = LinuxInput.KEY_FN_F;
    public static final short KEY_FN_S = LinuxInput.KEY_FN_S;
    public static final short KEY_FN_B = LinuxInput.KEY_FN_B;
    public static final short KEY_BRL_DOT1 = LinuxInput.KEY_BRL_DOT1;
    public static final short KEY_BRL_DOT2 = LinuxInput.KEY_BRL_DOT2;
    public static final short KEY_BRL_DOT3 = LinuxInput.KEY_BRL_DOT3;
    public static final short KEY_BRL_DOT4 = LinuxInput.KEY_BRL_DOT4;
    public static final short KEY_BRL_DOT5 = LinuxInput.KEY_BRL_DOT5;
    public static final short KEY_BRL_DOT6 = LinuxInput.KEY_BRL_DOT6;
    public static final short KEY_BRL_DOT7 = LinuxInput.KEY_BRL_DOT7;
    public static final short KEY_BRL_DOT8 = LinuxInput.KEY_BRL_DOT8;
    public static final short KEY_BRL_DOT9 = LinuxInput.KEY_BRL_DOT9;
    public static final short KEY_BRL_DOT10 = LinuxInput.KEY_BRL_DOT10;
    public static final short KEY_NUMERIC_0 = LinuxInput.KEY_NUMERIC_0;
    public static final short KEY_NUMERIC_1 = LinuxInput.KEY_NUMERIC_1;
    public static final short KEY_NUMERIC_2 = LinuxInput.KEY_NUMERIC_2;
    public static final short KEY_NUMERIC_3 = LinuxInput.KEY_NUMERIC_3;
    public static final short KEY_NUMERIC_4 = LinuxInput.KEY_NUMERIC_4;
    public static final short KEY_NUMERIC_5 = LinuxInput.KEY_NUMERIC_5;
    public static final short KEY_NUMERIC_6 = LinuxInput.KEY_NUMERIC_6;
    public static final short KEY_NUMERIC_7 = LinuxInput.KEY_NUMERIC_7;
    public static final short KEY_NUMERIC_8 = LinuxInput.KEY_NUMERIC_8;
    public static final short KEY_NUMERIC_9 = LinuxInput.KEY_NUMERIC_9;
    public static final short KEY_NUMERIC_STAR = LinuxInput.KEY_NUMERIC_STAR;
    public static final short KEY_NUMERIC_POUND = LinuxInput.KEY_NUMERIC_POUND;
    public static final short KEY_CAMERA_FOCUS = LinuxInput.KEY_CAMERA_FOCUS;
    public static final short KEY_WPS_BUTTON = LinuxInput.KEY_WPS_BUTTON;
    public static final short KEY_TOUCHPAD_TOGGLE = LinuxInput.KEY_TOUCHPAD_TOGGLE;
    public static final short KEY_TOUCHPAD_ON = LinuxInput.KEY_TOUCHPAD_ON;
    public static final short KEY_TOUCHPAD_OFF = LinuxInput.KEY_TOUCHPAD_OFF;
    public static final short KEY_CAMERA_ZOOMIN = LinuxInput.KEY_CAMERA_ZOOMIN;
    public static final short KEY_CAMERA_ZOOMOUT = LinuxInput.KEY_CAMERA_ZOOMOUT;
    public static final short KEY_CAMERA_UP = LinuxInput.KEY_CAMERA_UP;
    public static final short KEY_CAMERA_DOWN = LinuxInput.KEY_CAMERA_DOWN;
    public static final short KEY_CAMERA_LEFT = LinuxInput.KEY_CAMERA_LEFT;
    public static final short KEY_CAMERA_RIGHT = LinuxInput.KEY_CAMERA_RIGHT;
    public static final short BTN_TRIGGER_HAPPY = LinuxInput.BTN_TRIGGER_HAPPY;
    public static final short BTN_TRIGGER_HAPPY1 = LinuxInput.BTN_TRIGGER_HAPPY1;
    public static final short BTN_TRIGGER_HAPPY2 = LinuxInput.BTN_TRIGGER_HAPPY2;
    public static final short BTN_TRIGGER_HAPPY3 = LinuxInput.BTN_TRIGGER_HAPPY3;
    public static final short BTN_TRIGGER_HAPPY4 = LinuxInput.BTN_TRIGGER_HAPPY4;
    public static final short BTN_TRIGGER_HAPPY5 = LinuxInput.BTN_TRIGGER_HAPPY5;
    public static final short BTN_TRIGGER_HAPPY6 = LinuxInput.BTN_TRIGGER_HAPPY6;
    public static final short BTN_TRIGGER_HAPPY7 = LinuxInput.BTN_TRIGGER_HAPPY7;
    public static final short BTN_TRIGGER_HAPPY8 = LinuxInput.BTN_TRIGGER_HAPPY8;
    public static final short BTN_TRIGGER_HAPPY9 = LinuxInput.BTN_TRIGGER_HAPPY9;
    public static final short BTN_TRIGGER_HAPPY10 = LinuxInput.BTN_TRIGGER_HAPPY10;
    public static final short BTN_TRIGGER_HAPPY11 = LinuxInput.BTN_TRIGGER_HAPPY11;
    public static final short BTN_TRIGGER_HAPPY12 = LinuxInput.BTN_TRIGGER_HAPPY12;
    public static final short BTN_TRIGGER_HAPPY13 = LinuxInput.BTN_TRIGGER_HAPPY13;
    public static final short BTN_TRIGGER_HAPPY14 = LinuxInput.BTN_TRIGGER_HAPPY14;
    public static final short BTN_TRIGGER_HAPPY15 = LinuxInput.BTN_TRIGGER_HAPPY15;
    public static final short BTN_TRIGGER_HAPPY16 = LinuxInput.BTN_TRIGGER_HAPPY16;
    public static final short BTN_TRIGGER_HAPPY17 = LinuxInput.BTN_TRIGGER_HAPPY17;
    public static final short BTN_TRIGGER_HAPPY18 = LinuxInput.BTN_TRIGGER_HAPPY18;
    public static final short BTN_TRIGGER_HAPPY19 = LinuxInput.BTN_TRIGGER_HAPPY19;
    public static final short BTN_TRIGGER_HAPPY20 = LinuxInput.BTN_TRIGGER_HAPPY20;
    public static final short BTN_TRIGGER_HAPPY21 = LinuxInput.BTN_TRIGGER_HAPPY21;
    public static final short BTN_TRIGGER_HAPPY22 = LinuxInput.BTN_TRIGGER_HAPPY22;
    public static final short BTN_TRIGGER_HAPPY23 = LinuxInput.BTN_TRIGGER_HAPPY23;
    public static final short BTN_TRIGGER_HAPPY24 = LinuxInput.BTN_TRIGGER_HAPPY24;
    public static final short BTN_TRIGGER_HAPPY25 = LinuxInput.BTN_TRIGGER_HAPPY25;
    public static final short BTN_TRIGGER_HAPPY26 = LinuxInput.BTN_TRIGGER_HAPPY26;
    public static final short BTN_TRIGGER_HAPPY27 = LinuxInput.BTN_TRIGGER_HAPPY27;
    public static final short BTN_TRIGGER_HAPPY28 = LinuxInput.BTN_TRIGGER_HAPPY28;
    public static final short BTN_TRIGGER_HAPPY29 = LinuxInput.BTN_TRIGGER_HAPPY29;
    public static final short BTN_TRIGGER_HAPPY30 = LinuxInput.BTN_TRIGGER_HAPPY30;
    public static final short BTN_TRIGGER_HAPPY31 = LinuxInput.BTN_TRIGGER_HAPPY31;
    public static final short BTN_TRIGGER_HAPPY32 = LinuxInput.BTN_TRIGGER_HAPPY32;
    public static final short BTN_TRIGGER_HAPPY33 = LinuxInput.BTN_TRIGGER_HAPPY33;
    public static final short BTN_TRIGGER_HAPPY34 = LinuxInput.BTN_TRIGGER_HAPPY34;
    public static final short BTN_TRIGGER_HAPPY35 = LinuxInput.BTN_TRIGGER_HAPPY35;
    public static final short BTN_TRIGGER_HAPPY36 = LinuxInput.BTN_TRIGGER_HAPPY36;
    public static final short BTN_TRIGGER_HAPPY37 = LinuxInput.BTN_TRIGGER_HAPPY37;
    public static final short BTN_TRIGGER_HAPPY38 = LinuxInput.BTN_TRIGGER_HAPPY38;
    public static final short BTN_TRIGGER_HAPPY39 = LinuxInput.BTN_TRIGGER_HAPPY39;
    public static final short BTN_TRIGGER_HAPPY40 = LinuxInput.BTN_TRIGGER_HAPPY40;
    public static final short KEY_MIN_INTERESTING = LinuxInput.KEY_MIN_INTERESTING;
    public static final short KEY_MAX = LinuxInput.KEY_MAX;
    public static final short KEY_CNT = LinuxInput.KEY_CNT;
    public static final short REL_X = LinuxInput.REL_X;
    public static final short REL_Y = LinuxInput.REL_Y;
    public static final short REL_Z = LinuxInput.REL_Z;
    public static final short REL_RX = LinuxInput.REL_RX;
    public static final short REL_RY = LinuxInput.REL_RY;
    public static final short REL_RZ = LinuxInput.REL_RZ;
    public static final short REL_HWHEEL = LinuxInput.REL_HWHEEL;
    public static final short REL_DIAL = LinuxInput.REL_DIAL;
    public static final short REL_WHEEL = LinuxInput.REL_WHEEL;
    public static final short REL_MISC = LinuxInput.REL_MISC;
    public static final short REL_MAX = LinuxInput.REL_MAX;
    public static final short REL_CNT = LinuxInput.REL_CNT;
    public static final short ABS_X = LinuxInput.ABS_X;
    public static final short ABS_Y = LinuxInput.ABS_Y;
    public static final short ABS_Z = LinuxInput.ABS_Z;
    public static final short ABS_RX = LinuxInput.ABS_RX;
    public static final short ABS_RY = LinuxInput.ABS_RY;
    public static final short ABS_RZ = LinuxInput.ABS_RZ;
    public static final short ABS_THROTTLE = LinuxInput.ABS_THROTTLE;
    public static final short ABS_RUDDER = LinuxInput.ABS_RUDDER;
    public static final short ABS_WHEEL = LinuxInput.ABS_WHEEL;
    public static final short ABS_GAS = LinuxInput.ABS_GAS;
    public static final short ABS_BRAKE = LinuxInput.ABS_BRAKE;
    public static final short ABS_HAT0X = LinuxInput.ABS_HAT0X;
    public static final short ABS_HAT0Y = LinuxInput.ABS_HAT0Y;
    public static final short ABS_HAT1X = LinuxInput.ABS_HAT1X;
    public static final short ABS_HAT1Y = LinuxInput.ABS_HAT1Y;
    public static final short ABS_HAT2X = LinuxInput.ABS_HAT2X;
    public static final short ABS_HAT2Y = LinuxInput.ABS_HAT2Y;
    public static final short ABS_HAT3X = LinuxInput.ABS_HAT3X;
    public static final short ABS_HAT3Y = LinuxInput.ABS_HAT3Y;
    public static final short ABS_PRESSURE = LinuxInput.ABS_PRESSURE;
    public static final short ABS_DISTANCE = LinuxInput.ABS_DISTANCE;
    public static final short ABS_TILT_X = LinuxInput.ABS_TILT_X;
    public static final short ABS_TILT_Y = LinuxInput.ABS_TILT_Y;
    public static final short ABS_TOOL_WIDTH = LinuxInput.ABS_TOOL_WIDTH;
    public static final short ABS_VOLUME = LinuxInput.ABS_VOLUME;
    public static final short ABS_MISC = LinuxInput.ABS_MISC;
    public static final short ABS_MT_SLOT = LinuxInput.ABS_MT_SLOT;
    public static final short ABS_MT_TOUCH_MAJOR = LinuxInput.ABS_MT_TOUCH_MAJOR;
    public static final short ABS_MT_TOUCH_MINOR = LinuxInput.ABS_MT_TOUCH_MINOR;
    public static final short ABS_MT_WIDTH_MAJOR = LinuxInput.ABS_MT_WIDTH_MAJOR;
    public static final short ABS_MT_WIDTH_MINOR = LinuxInput.ABS_MT_WIDTH_MINOR;
    public static final short ABS_MT_ORIENTATION = LinuxInput.ABS_MT_ORIENTATION;
    public static final short ABS_MT_POSITION_X = LinuxInput.ABS_MT_POSITION_X;
    public static final short ABS_MT_POSITION_Y = LinuxInput.ABS_MT_POSITION_Y;
    public static final short ABS_MT_TOOL_TYPE = LinuxInput.ABS_MT_TOOL_TYPE;
    public static final short ABS_MT_BLOB_ID = LinuxInput.ABS_MT_BLOB_ID;
    public static final short ABS_MT_TRACKING_ID = LinuxInput.ABS_MT_TRACKING_ID;
    public static final short ABS_MT_PRESSURE = LinuxInput.ABS_MT_PRESSURE;
    public static final short ABS_MT_DISTANCE = LinuxInput.ABS_MT_DISTANCE;
    public static final short ABS_MAX = LinuxInput.ABS_MAX;
    public static final short ABS_CNT = LinuxInput.ABS_CNT;
    public static final short SW_LID = LinuxInput.SW_LID;
    public static final short SW_TABLET_MODE = LinuxInput.SW_TABLET_MODE;
    public static final short SW_HEADPHONE_INSERT = LinuxInput.SW_HEADPHONE_INSERT;
    public static final short SW_RFKILL_ALL = LinuxInput.SW_RFKILL_ALL;
    public static final short SW_RADIO = LinuxInput.SW_RADIO;
    public static final short SW_MICROPHONE_INSERT = LinuxInput.SW_MICROPHONE_INSERT;
    public static final short SW_DOCK = LinuxInput.SW_DOCK;
    public static final short SW_LINEOUT_INSERT = LinuxInput.SW_LINEOUT_INSERT;
    public static final short SW_JACK_PHYSICAL_INSERT = LinuxInput.SW_JACK_PHYSICAL_INSERT;
    public static final short SW_VIDEOOUT_INSERT = LinuxInput.SW_VIDEOOUT_INSERT;
    public static final short SW_CAMERA_LENS_COVER = LinuxInput.SW_CAMERA_LENS_COVER;
    public static final short SW_KEYPAD_SLIDE = LinuxInput.SW_KEYPAD_SLIDE;
    public static final short SW_FRONT_PROXIMITY = LinuxInput.SW_FRONT_PROXIMITY;
    public static final short SW_ROTATE_LOCK = LinuxInput.SW_ROTATE_LOCK;
    public static final short SW_LINEIN_INSERT = LinuxInput.SW_LINEIN_INSERT;
    public static final short SW_MAX = LinuxInput.SW_MAX;
    public static final short SW_CNT = LinuxInput.SW_CNT;
    public static final short MSC_SERIAL = LinuxInput.MSC_SERIAL;
    public static final short MSC_PULSELED = LinuxInput.MSC_PULSELED;
    public static final short MSC_GESTURE = LinuxInput.MSC_GESTURE;
    public static final short MSC_RAW = LinuxInput.MSC_RAW;
    public static final short MSC_SCAN = LinuxInput.MSC_SCAN;
    public static final short MSC_MAX = LinuxInput.MSC_MAX;
    public static final short MSC_CNT = LinuxInput.MSC_CNT;
    public static final short LED_NUML = LinuxInput.LED_NUML;
    public static final short LED_CAPSL = LinuxInput.LED_CAPSL;
    public static final short LED_SCROLLL = LinuxInput.LED_SCROLLL;
    public static final short LED_COMPOSE = LinuxInput.LED_COMPOSE;
    public static final short LED_KANA = LinuxInput.LED_KANA;
    public static final short LED_SLEEP = LinuxInput.LED_SLEEP;
    public static final short LED_SUSPEND = LinuxInput.LED_SUSPEND;
    public static final short LED_MUTE = LinuxInput.LED_MUTE;
    public static final short LED_MISC = LinuxInput.LED_MISC;
    public static final short LED_MAIL = LinuxInput.LED_MAIL;
    public static final short LED_CHARGING = LinuxInput.LED_CHARGING;
    public static final short LED_MAX = LinuxInput.LED_MAX;
    public static final short LED_CNT = LinuxInput.LED_CNT;
    public static final short REP_DELAY = LinuxInput.REP_DELAY;
    public static final short REP_PERIOD = LinuxInput.REP_PERIOD;
    public static final short REP_MAX = LinuxInput.REP_MAX;
    public static final short REP_CNT = LinuxInput.REP_CNT;
    public static final short SND_CLICK = LinuxInput.SND_CLICK;
    public static final short SND_BELL = LinuxInput.SND_BELL;
    public static final short SND_TONE = LinuxInput.SND_TONE;
    public static final short SND_MAX = LinuxInput.SND_MAX;
    public static final short SND_CNT = LinuxInput.SND_CNT;
    public static final short ID_BUS = LinuxInput.ID_BUS;
    public static final short ID_VENDOR = LinuxInput.ID_VENDOR;
    public static final short ID_PRODUCT = LinuxInput.ID_PRODUCT;
    public static final short ID_VERSION = LinuxInput.ID_VERSION;
    public static final short BUS_PCI = LinuxInput.BUS_PCI;
    public static final short BUS_ISAPNP = LinuxInput.BUS_ISAPNP;
    public static final short BUS_USB = LinuxInput.BUS_USB;
    public static final short BUS_HIL = LinuxInput.BUS_HIL;
    public static final short BUS_BLUETOOTH = LinuxInput.BUS_BLUETOOTH;
    public static final short BUS_VIRTUAL = LinuxInput.BUS_VIRTUAL;
    public static final short BUS_ISA = LinuxInput.BUS_ISA;
    public static final short BUS_I8042 = LinuxInput.BUS_I8042;
    public static final short BUS_XTKBD = LinuxInput.BUS_XTKBD;
    public static final short BUS_RS232 = LinuxInput.BUS_RS232;
    public static final short BUS_GAMEPORT = LinuxInput.BUS_GAMEPORT;
    public static final short BUS_PARPORT = LinuxInput.BUS_PARPORT;
    public static final short BUS_AMIGA = LinuxInput.BUS_AMIGA;
    public static final short BUS_ADB = LinuxInput.BUS_ADB;
    public static final short BUS_I2C = LinuxInput.BUS_I2C;
    public static final short BUS_HOST = LinuxInput.BUS_HOST;
    public static final short BUS_GSC = LinuxInput.BUS_GSC;
    public static final short BUS_ATARI = LinuxInput.BUS_ATARI;
    public static final short BUS_SPI = LinuxInput.BUS_SPI;
    public static final short MT_TOOL_FINGER = LinuxInput.MT_TOOL_FINGER;
    public static final short MT_TOOL_PEN = LinuxInput.MT_TOOL_PEN;
    public static final short MT_TOOL_MAX = LinuxInput.MT_TOOL_MAX;
    public static final short FF_STATUS_STOPPED = LinuxInput.FF_STATUS_STOPPED;
    public static final short FF_STATUS_PLAYING = LinuxInput.FF_STATUS_PLAYING;
    public static final short FF_STATUS_MAX = LinuxInput.FF_STATUS_MAX;
    public static final short FF_RUMBLE = LinuxInput.FF_RUMBLE;
    public static final short FF_PERIODIC = LinuxInput.FF_PERIODIC;
    public static final short FF_CONSTANT = LinuxInput.FF_CONSTANT;
    public static final short FF_SPRING = LinuxInput.FF_SPRING;
    public static final short FF_FRICTION = LinuxInput.FF_FRICTION;
    public static final short FF_DAMPER = LinuxInput.FF_DAMPER;
    public static final short FF_INERTIA = LinuxInput.FF_INERTIA;
    public static final short FF_RAMP = LinuxInput.FF_RAMP;
    public static final short FF_EFFECT_MIN = LinuxInput.FF_EFFECT_MIN;
    public static final short FF_EFFECT_MAX = LinuxInput.FF_EFFECT_MAX;
    public static final short FF_SQUARE = LinuxInput.FF_SQUARE;
    public static final short FF_TRIANGLE = LinuxInput.FF_TRIANGLE;
    public static final short FF_SINE = LinuxInput.FF_SINE;
    public static final short FF_SAW_UP = LinuxInput.FF_SAW_UP;
    public static final short FF_SAW_DOWN = LinuxInput.FF_SAW_DOWN;
    public static final short FF_CUSTOM = LinuxInput.FF_CUSTOM;
    public static final short FF_WAVEFORM_MIN = LinuxInput.FF_WAVEFORM_MIN;
    public static final short FF_WAVEFORM_MAX = LinuxInput.FF_WAVEFORM_MAX;
    public static final short FF_GAIN = LinuxInput.FF_GAIN;
    public static final short FF_AUTOCENTER = LinuxInput.FF_AUTOCENTER;
    public static final short FF_MAX = LinuxInput.FF_MAX;
    public static final short FF_CNT = LinuxInput.FF_CNT;

}
